from pyOwlBoard import client
import os

ob_client = client.OwlBoardClient('https://owlboard.info', os.getenv("OWLBOARD_KEY"))

def find_gw_trains_by_headcode(headcode, date):
    train_list = ob_client.get_trains_by_headcode(headcode, date)

    # Filter for 'gw & hx' services only
    for train in train_list[:]:
        if train['operator'] not in ('GW', 'HX'):
            train_list.remove(train)

    # Get details for any services remaining in list
    train_detail_list = []
    for train in train_list:
        train_detail = ob_client.get_trains_by_trainUid(train['trainUid'], date)
        train_detail_list.append(train_detail)

    return train_detail_list

# Organize list of trains removing unimportant information
def organise_trains(trains):
    organised_trains = []
    for train in trains:
        organised_train = {
            'diagram_date': train['schedule_date'],
            'train_headcode': train['schedule_headcode'],
            'diagram_time1': train['schedule_time1'],
            'diagram_time2': train['schedule_time2'],
            'diagram_pis_code': train['schedule_pis_code'],
            'timetable_entries': rationalise_timetable_entry(train['timetable_entry']),
        }

        organised_trains.append(organised_train)
    return organised_trains

# Convert timetable entry into a list of CRS Codes for PIS identification
def rationalise_timetable_entry(timetable_entries):
    rationalised_entries = []
    for timetable_entry in timetable_entries:
        if timetable_entry['stpIndicator'] == "C": continue
        entry = {
            'stpIndicator': timetable_entry['stpIndicator'],
            'operator': timetable_entry['operator'],
            'trainUid': timetable_entry['trainUid'],
            'scheduleStart': timetable_entry['scheduleStart'],
            'scheduleEnd': timetable_entry['scheduleEnd'],
            'daysRun': timetable_entry['daysRun'],
            'trainStartTime': timetable_entry['stops'][0].get('wttDeparture') or timetable_entry['stops'][0].get('publicDeparture'),
            'stops': create_crs_list_from_stops(timetable_entry['stops']),
        }

        rationalised_entries.append(entry)
    return rationalised_entries

# Work through list of stops, and create a list of CRS codes for public stops
def create_crs_list_from_stops(stops):
    crs_list = []
    for stop in stops:
        if not stop['isPublic']:
            continue
        try:
            loc_ref = ob_client.get_loc_ref_codes_by_tiploc(stop['tiploc'])
            crs_list.append(loc_ref[0]['3ALPHA'])
        except Exception as e:
            print("Unable to load 3ALPHA for: ", stop['tiploc'])
            crs_list.append("---")
    return crs_list